#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# Name:	Color By
#   By:	sdmitch
#Usage:	Add color to faces/edges based on direction, slope, layer, or plane.
#	Note:	Color by Direction in 22,5deg increments, ignores faces on the XY plane
#			  Color by Slope in 10deg increments is same for north and south hemisphere.
#       Color by Plane for a given value with tolerance, enter high tolerance for all values.
#					assigns a color that is the combination of axis colors,ie.XY=red+green=yellow
# Date:	Aug 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
	sub = UI.menu('Plugins').add_submenu('Color By:')
	sub.add_item('Direction') { SDM_CBDSLP.sdm_color_by_direction }
	sub.add_item('Slope') { SDM_CBDSLP.sdm_color_by_slope }
	sub.add_item('Layer') { SDM_CBDSLP.sdm_color_by_layer }
	sub.add_item('Plane') { SDM_CBDSLP.sdm_color_by_plane }
end
#
file_loaded(filename)
#
module SDM_CBDSLP

	def self.sdm_color_by_direction
		mod = Sketchup.active_model
		ent = mod.active_entities
		sel = mod.selection
		faces=[];edges=[]
		if !sel.empty?
			mod.start_operation "Color By Direction"
			faces = sel.find_all{|f| f.is_a?(Sketchup::Face)&& f.normal.z.abs < 0.99}
			edges = sel.find_all{|e| e.is_a?(Sketchup::Edge)&& e.line[1].z.abs < 0.99}
			grps = sel.find_all{|g| g.is_a?(Sketchup::Group)}
			cmps = sel.find_all{|c| c.is_a?(Sketchup::ComponentInstance)}
			for g in grps
				for ge in g.entities
					faces<<ge if ge.is_a?(Sketchup::Face)
					edges<<ge if ge.is_a?(Sketchup::Edge)
				end
			end
			for c in cmps
				for ce in c.definition.entities
					faces<<ce if ce.is_a?(Sketchup::Face)
					edges<<ce if ce.is_a?(Sketchup::Edge)
				end
			end
			for f in faces
				### rotate to vertical to get direction
				norm=f.normal; rpnt=f.bounds.center; axis=norm.axes[0]
				vang=Math.asin(norm.z)
				tr=Geom::Transformation.rotation(rpnt,axis,vang)
				norm.transform! tr; 
				f.material = f.back_material = self.direction_color(norm)
			end
			for e in edges
				p1=e.start.position;p2=e.end.position
				p2.z=p1.z;vec=p1.vector_to p2;vec.normalize!
				e.material=self.direction_color(vec)
			end
			sel.clear; mod.commit_operation
		else
			UI.messagebox "Nothing Selected"
		end
	end
###
	def self.sdm_color_by_slope
###
		mod = Sketchup.active_model
		ent = mod.active_entities
		sel = mod.selection
		if !sel.empty?
			faces = sel.find_all{|f| f.is_a?(Sketchup::Face)}
			edges = sel.find_all{|e| e.is_a?(Sketchup::Edge)}
			grps = sel.find_all{|g| g.is_a?(Sketchup::Group)}
			cmps = sel.find_all{|c| c.is_a?(Sketchup::ComponentInstance)}
			mod.start_operation "Color By Slope"
			for g in grps
				for ge in g.entities
					faces<<ge if ge.is_a?(Sketchup::Face)
					edges<<ge if ge.is_a?(Sketchup::Edge)
				end
			end
			for c in cmps
				for ce in c.definition.entities
					faces<<ce if ce.is_a?(Sketchup::Face)
					edges<<ce if ce.is_a?(Sketchup::Edge)
				end
			end
			for f in faces
				norm = f.normal; 
				f.material = f.back_material = self.slope_color(norm)
			end#for
			for e in edges
				vec=e.line[1]; 
				e.material = self.slope_color(vec)
			end
			sel.clear
			mod.commit_operation
		else
			UI.messagebox "Nothing Selected"
		end
	end
###
	def self.sdm_color_by_layer
###
		mod=Sketchup.active_model
		ent=mod.active_entities
		sel=mod.selection
		sel=ent if sel.empty?
		lyr=mod.layers
		lnm=[];lyr.each{|l| lnm<<l.name}
		cnm=['aqua','blue','brown','coral','cyan','gold','green','khaki','lime','magenta','purple','red','teal','yellow']
		opt1=lnm.join("|");opt2=cnm.join("|")
		ans=inputbox(["Layer:","Color:"],[lnm[0],cnm[0]],[opt1,opt2],"Color By Layer")
		return nil if !ans
		lay=ans[0];clr=ans[1]
		sel.each{|e|
			if e.is_a?(Sketchup::Group)
				e.entities.each{|ge| 
					if ge.layer.name == lay
						if ge.is_a?(Sketchup::Face)
							ge.material=ge.back_material=clr 
						elsif ge.is_a?(Sketchup::Edge)
							ge.material=clr
						end
					end
				}
			elsif e.is_a?(Sketchup::ComponentInstance)
				e.definition.entities.each{|ce|
					if ce.layer.name == lay
						if ce.is_a?(Sketchup::Face)
							ce.material=ce.back_material=clr 
						elsif ce.is_a?(Sketchup::Edge)
							ce.material=clr
						end
					end
				}
			elsif e.layer.name == lay
				if e.is_a?(Sketchup::Face) 
					e.material=e.back_material=clr 
				elsif e.is_a?(Sketchup::Edge)
					e.material=clr 
				end
			end
		}
	end
###
	def self.sdm_color_by_plane
###
		mod = Sketchup.active_model
		ent = mod.active_entities
		sel = mod.selection
		if !sel.empty?
			cnm=['aqua','blue','brown','coral','cyan','gold','green','khaki','lime','magenta','purple','red','teal','yellow']
			opt1=cnm.join("|");prompts=["Select Plane:","Select Color:","Value to select:","Value tolerance:"]
			ans=UI.inputbox(prompts,["XY","red",0.feet,0.125.inch],["XY|XZ|YZ",opt1],"Color by Plane")
			if !ans then return end
			plane=ans[0].to_s;clr=ans[1].to_s;val=ans[2].to_l;tol=ans[3].to_l
			case plane
				when "XY" #same Z
					sel.each{|e|
						if e.is_a?(Sketchup::ComponentInstance)
							pt=e.origin
							e.definition.entities.each{|ce| 
								if ce.is_a?(Sketchup::Edge)
									 if ce.start.position.z==ce.end.position.z
										if (ce.start.position.z - val).abs <= tol
											ce.material=clr;#[255,255,0];#yellow
										end
									 end
								end
							}
						elsif e.is_a?(Sketchup::Group)
							e.entities.each{|ge| 
								if ge.is_a?(Sketchup::Edge)
									 if ge.start.position.z==ge.end.position.z
										if (ge.start.position.z - val).abs <= tol
											ge.material=clr;#[255,255,0];#yellow
										end
									 end
								end
							}
						elsif e.is_a?(Sketchup::Edge)
							 if e.start.position.z==e.end.position.z
								if (e.start.position.z - val).abs <= tol
									e.material=clr;#[255,255,0];#yellow
                                end
							 end
						end
					}
				when "XZ" # same Y
					sel.each{|e|
						if e.is_a?(Sketchup::ComponentInstance)
							e.definition.entities.each{|ce| 
								if ce.is_a?(Sketchup::Edge)
									 if ce.start.position.y==ce.end.position.y
										if (ce.start.position.y - val).abs <= tol
											ce.material=clr;#[255,255,0];#yellow
										end
									 end
								end
							}
						elsif e.is_a?(Sketchup::Group)
							e.entities.each{|ge| 
								if ge.is_a?(Sketchup::Edge)
									 if ge.start.position.y==ge.end.position.y
										if (ge.start.position.y - val).abs <= tol
											ge.material=clr;#[255,255,0];#yellow
										end
									 end
								end
							}
						elsif e.is_a?(Sketchup::Edge)
							 if e.start.position.y==e.end.position.y
								if (e.start.position.y - val).abs <= tol
									e.material=clr;#[255,255,0];#yellow
                                end
							 end
						end
					}
				when "YZ" # same X
					sel.each{|e|
						if e.is_a?(Sketchup::ComponentInstance)
							e.definition.entities.each{|ce| 
								if ce.is_a?(Sketchup::Edge)
									 if ce.start.position.x==ce.end.position.x
										if (ce.start.position.x - val).abs <= tol
											ce.material=clr;#[255,255,0];#yellow
										end
									 end
								end
							}
						elsif e.is_a?(Sketchup::Group)
							e.entities.each{|ge| 
								if ge.is_a?(Sketchup::Edge)
									 if ge.start.position.x==ge.end.position.x
										if (ge.start.position.x - val).abs <= tol
											ge.material=clr;#[255,255,0];#yellow
										end
									 end
								end
							}
						elsif e.is_a?(Sketchup::Edge)
							 if e.start.position.x==e.end.position.x
								if (e.start.position.x - val).abs <= tol
									e.material=clr;#[255,255,0];#yellow
                                end
							 end
						end
					}
			end
			sel.clear
		else
			UI.messagebox "Nothing selected"
		end
	end

	def self.direction_color(v)
		v.y = v.y > 1.0 ? 1.0 : v.y
		angle=Math.acos(v.y).radians
		angle=360.0-angle if v.x < 0.0
		### change these test ranges and/or color to suit...
		case angle
			when 11.25...33.75;color=[255,255,0];			#yellow
			when 33.75...56.25;color=[138,43,226];			#blueviolet
			when 56.25...78.75;color=[218,112,214];		#orchid
			when 78.75...101.25;color=[255,215,0];			#gold
			when 101.25...123.75;color=[0,255,0];			#green
			when 123.75...146.25;color=[165,42,42];		#brown
			when 146.25...168.75;color=[255,128,80];		#coral
			when 168.75...191.25;color=[128,128,0];		#olive
			when 191.25...213.75;color=[75,0,130];			#indigo
			when 213.75...236.25;color=[105,105,105];	#darkgrey
			when 236.25...258.75;color=[0,0,255];			#blue
			when 258.75...281.25;color=[211,211,211];	#lightgrey
			when 281.25...303.75;color=[255,0,255];		#fuchsia
			when 303.75...326.25;color=[255,165,0];		#orange
			when 326.25...348.75;color=[0,255,255];		#aqua/cyan
			else color=[255,0,0];											#red
		end#case
	end

	def self.slope_color(v)
		angle=Math.acos(v.z.abs).radians
		case angle
			when   0.0...10.0;color=[0,255,0]
			when 10.0...20.0;color=[255,255,0]
			when 20.0...30.0;color=[0,255,255]
			when 30.0...40.0;color=[128,128,0]
			when 40.0...50.0;color=[0,0,255]
			when 50.0...60.0;color=[170,244,97]
			when 60.0...70.0;color=[255,128,0]
			when 70.0...80.0;color=[255,0,255]
			else color=[255,0,0]
		end#case
	end
end